@@ -1,12 +1,12 @@
 #!/usr/bin/env python
-from __future__ import generators
 from SimPy.MonitorTest import *
 from SimPy.SimulationTrace  import *
+
 import unittest
 from random import random
-# $Revision: 1.1.1.11 $ $Date: 2006/06/06 15:39:11 $ 
+# $Revision: 1.1.1.13 $ $Date: 2007/01/08 14:07:34 $ 
 """testSimPyTrace.py
-SimPy version 1.7.1 
+SimPy version 1.8
 Unit tests for SimulationTrace.
 
 **Change history:**
@@ -30,9 +30,16 @@ Unit tests for SimulationTrace.
              supported for initialBuffered
 # 2006 05 16 added tests for Store and Level to test basic Producer/Consumer 
              principles
+# 2006 10 16 added tests for compound get statement (Unmonitored Store/Level)
+# 2006 10 17 added tests for compound put statement (Unmonitored Store/Level)
+# 2007 01 08 added tests for monitoring of Store/Level with compound get/put
+# 2007 01 08 added test for Store with filter function
+
+#'$Revision: 1.1.1.13 $ $Date: 2007/01/08 14:07:34 $ kgm'
+
 """
-__version__ = '1.7.1 Jun 2006'
-Revision = '$Revision: 1.1.1.11 $'
+__version__ = '1.8 $Revision: 1.1.1.13 $ $Date: 2007/01/08 14:07:34 $ '
+print "testSimPyTrace.py %s"%__version__
 
 ## -------------------------------------------------------------
 ##                    TEST SIMULATION
@@ -1485,7 +1492,7 @@ class makeLevelTestcase(unittest.TestCase):
                                     %doneList
                                     
     def testConPriorProdM(self):
-        """Level: ests put/get in multiple Producer/Consumer scenario, with
+        """Level: tests put/get in multiple Producer/Consumer scenario, with
         Consumers having different priorities.
         How: Consumers forced to queue; all after first should be done in
         priority order
@@ -1517,7 +1524,6 @@ def makeLevelSuite():
                     testConPriorProdM]) 
     return suite
 
-
 ## ------------------------------------------------------------------
 ##             TEST "yield get,self,store,whatToGet" and 
 ##                  "yield put,self,store,whatToPut"
@@ -1603,6 +1609,36 @@ class ConsumerPrincS(Process):
                 
 class WidgetPrinc:
     pass
+    
+class FilterConsumer(Process):
+    """Used in testBufferFilter"""
+    class Widget:
+        def __init__(self,weighs):
+            self.weight=weighs
+            
+    def getItems(self,store,a,b):
+        """get all items with weight between a and b"""
+        def between_a_and_b(buf):
+            res=[]
+            for item in buf:
+                if a<item.weight<b:
+                    res.append(item)
+                    
+        all=store.buffered
+        yield get,self,store,between_a_and_b
+        "All retrieved items weight in range?"
+        for it in self.got:
+            assert a<it.weight<b,"weight %s not in range %s..%s"\
+                                 %(it.weight,a,b)
+        "Any item fitting filter pred left in buffer?"
+        for it in store.buffer:
+            assert not (a<it.weight<b),\
+                "item left in buffer which fits filter (%s<%s<%s)"\
+                %(a,it.weight,b)
+        "All items either in store.buffer of self.got?"
+        for it in all:
+            assert (it in self.buffer) or (it in self.got),\
+                   "item w. weight %s neither in store nor in got"%it.weight 
 
 ### End classes for testConPrinciple (Store) ###
 
@@ -1663,9 +1699,10 @@ class makeStoreTestcase(unittest.TestCase):
         c=ConsumerWidget()
         activate(c,c.consume(buffer))
         simulate(until=100)
-        assert ProducerWidget.produced-ConsumerWidget.consumed==buffer.nrBuffered,\
-            "items produced/consumed/buffered do not tally: %s %s %s"\
-            %(ProducerWidget.produced,ConsumerWidget.consumed,buffer.nrBuffered)
+        assert \
+           ProducerWidget.produced-ConsumerWidget.consumed==buffer.nrBuffered,\
+           "items produced/consumed/buffered do not tally: %s %s %s"\
+           %(ProducerWidget.produced,ConsumerWidget.consumed,buffer.nrBuffered)
         
     def testConProdM(self):
         """Store: tests put/get in multiple Producer/Consumer scenario"""
@@ -1721,8 +1758,8 @@ class makeStoreTestcase(unittest.TestCase):
         p=ProducerWidget()
         activate(p,p.produce1(buffer=buffer))
         simulate(until=100)
-        assert doneList==["3","2","1","0"],"gets were not done in priority order: %s"\
-                                    %doneList
+        assert doneList==["3","2","1","0"],\
+              "gets were not done in priority order: %s"%doneList
                                     
     def testBufferSort(self):
         """Tests the optional sorting of theBuffer by applying a user-defined
@@ -1739,6 +1776,18 @@ class makeStoreTestcase(unittest.TestCase):
         simulate(until=10)
         assert gotten==[1,2,3,4,5,6,7,8,9],"sort wrong: %s"%gotten
         
+    def testBufferFilter(self):
+        """Tests get from a Store with a filter function
+        """
+        initialize()
+        ItClass=FilterConsumer.Widget
+        all=[ItClass(1),ItClass(4),ItClass(6),ItClass(12)]
+        st=Store(initialBuffered=all)
+        fc=FilterConsumer()
+        minw=2;maxw=10
+        activate(fc,fc.getItems(store=st,a=minw,b=maxw))
+        simulate(until=1)
+        
 def makeStoreSuite():
     suite = unittest.TestSuite()
     testStatic = makeStoreTestcase("testStatic")
@@ -1748,11 +1797,395 @@ def makeStoreSuite():
     testConProdPriorM=makeStoreTestcase("testConProdPriorM")
     testConPriorProdM=makeStoreTestcase("testConPriorProdM")
     testBufferSort=makeStoreTestcase("testBufferSort")
+    testBufferFilter=makeStoreTestcase("testBufferFilter")
     suite.addTests([testStatic,testConProdPrinciple,testConProd1,
                     testConProdM,testConProdPriorM,
-                    testConPriorProdM,testBufferSort]) 
+                    testConPriorProdM,testBufferSort,
+                    testBufferFilter]) 
     return suite
 
+## ------------------------------------------------------------------
+##             
+##  Store: Tests for compound get/put                
+##             
+## ------------------------------------------------------------------
+class TBT(Process):
+    """Store: For testBasicTime"""
+    def tbt(self,store):
+        yield get,self,store,1
+        assert self.got,"Did not get Item"
+        yield (get,self,store,1),(hold,self,5)
+        if self.acquired(store):
+            assert len(self.got)==1,"did not get 1 Item"
+        else:
+            assert not self.got and now()==5 and not store.getQ,\
+                   "time renege not working"
+        
+class TBE(Process):
+    """Store: For testBasicEvent"""
+    def tbe(self,store,trigger):
+        yield get,self,store,1
+        assert self.got,"Did not get Item"
+        yield (get,self,store,1),(waitevent,self,trigger)
+        if self.acquired(store):
+            assert False, "should have reneged"
+        else:
+            assert self.eventsFired[0]==trigger and now()==5 \
+                and not store.getQ,"event renege not working"
+        
+class TBEtrigger(Process):
+    """Store: For testBasicEvent"""
+    def fire(self,trigger):
+        yield hold,self,5
+        trigger.signal()
+    
+class makeStoreCompTestcase(unittest.TestCase):
+    """Store: Testcase for compound get statements"""
+    
+class TBTput(Process):
+    """Store: for testBasicTimePut"""
+    def tbt(self,store):
+        class Item:pass
+        yield (put,self,store,[Item()]),(hold,self,4)
+        if self.stored(store):
+            assert store.nrBuffered==1 and not store.putQ,\
+                   "put did not execute"
+        else:
+            assert False,"should not have reneged"
+        yield (put,self,store,[Item()]),(hold,self,5)
+        if self.stored(store):
+            assert False,"should have reneged"
+        else:
+            assert store.nrBuffered==1 and not store.putQ,\
+                   "renege not working correctly"   
+
+class TBEput(Process):
+    """Store: for testBasicEventPut"""
+    def tbe(self,store,trigger):
+        class Item:pass
+        yield (put,self,store,[Item()]),(waitevent,self,trigger)
+        if self.stored(store):
+            assert store.nrBuffered==1 and not store.putQ,\
+                   "put did not execute"
+        else:
+            assert False,"should have not have reneged"
+        yield (put,self,store,[Item()]),(waitevent,self,trigger)
+        if self.stored(store):
+            assert False,"should have reneged"
+        else:
+            assert now()==5 and self.eventsFired[0]==trigger\
+                   and not store.putQ,"renege not working correctly"   
+
+class TBEtriggerPut(Process):
+    """Store: For testBasicEventPut"""
+    def fire(self,trigger):
+        yield hold,self,5
+        trigger.signal()            
+    
+class makeStoreCompTestcase(unittest.TestCase):
+    """Store: Testcase for compound get statements"""
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (get,self,store),(hold,self,time)"
+    ##                   == timeout renege
+    ##             for both unmonitored and monitored Stores
+    ## ------------------------------------------------------------------
+
+    def testBasicTime(self):
+        """Store (unmonitored): 
+        test 'yield (get,self,store),(hold,self,timeout)"""
+        class Item:pass
+        initialize()
+        st=Store(initialBuffered=[Item()])
+        t=TBT()
+        activate(t,t.tbt(store=st))
+        simulate(until=10)
+        
+
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (put,self,store),(hold,self,time)"
+    ##                   == timeout renege
+    ##             for both unmonitored and monitored Stores
+    ## ------------------------------------------------------------------
+    def testBasicTimePut(self):
+        """Store (unmonitored): 
+        test 'yield (put,self,store),(hold,self,time)"""
+        initialize()
+        st=Store(capacity=1)
+        t=TBTput()
+        activate(t,t.tbt(store=st))
+        simulate(until=10)
+        
+    def testBasicTimePutM(self):
+        """Store (monitored): 
+        test monitors with 'yield (put,self,store),(hold,self,time)"""
+        initialize()
+        st=Store(capacity=1,monitored=True)
+        t=TBTput()
+        activate(t,t.tbt(store=st))
+        simulate(until=10)
+        #First put succeeds, second attempt reneges at t=5?
+        assert st.putQMon==[[0,0],[0,1],[5,0]],"putQMon wrong: %s"\
+                                               %st.putQMon
+        #First Item goes into buffer at t=0, second not (renege)?
+        assert st.bufferMon==[[0,0],[0,1]],"bufferMon wrong: %s"%st.bufferMon
+                                            
+        
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (get,self,store),(waitevent,self,event)"
+    ##                   == event renege
+    ##             for both unmonitored and monitored Stores
+    ## ------------------------------------------------------------------ 
+    def testBasicEvent(self):
+        """Store (unmonitored): 
+        test 'yield (get,self,store),(waitevent,self,event)"""
+        class Item:pass
+        initialize()
+        st=Store(initialBuffered=[Item()])
+        trig=SimEvent()
+        t=TBE()
+        activate(t,t.tbe(store=st,trigger=trig))
+        tr=TBEtrigger()
+        activate(tr,tr.fire(trigger=trig))
+        simulate(until=10) 
+        
+
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (put,self,store),(waitevent,self,event)"
+    ##                   == event renege
+    ##             for both unmonitored and monitored Stores
+    ## ------------------------------------------------------------------ 
+    def testBasicEventPut(self):
+        """Store (unmonitored): 
+        test 'yield (put,self,store),(waitevent,self,event)"""
+        initialize()
+        s=SimEvent()
+        store=Store(capacity=1)
+        t=TBEtriggerPut()
+        activate(t,t.fire(trigger=s))
+        tb=TBEput()
+        activate(tb,tb.tbe(store=store,trigger=s))
+        simulate(until=10)
+
+    def testBasicEventPutM(self):
+        """Store (monitored): 
+        test monitors with 'yield (put,self,store),(waitevent,self,event)""" 
+        initialize()
+        s=SimEvent()
+        st=Store(capacity=1,monitored=True)
+        t=TBEtriggerPut()
+        activate(t,t.fire(trigger=s))
+        tb=TBEput()
+        activate(tb,tb.tbe(store=st,trigger=s))
+        simulate(until=10)
+        #First put succeeds, second attempt reneges at t=5?
+        assert st.putQMon==[[0,0],[0,1],[5,0]],"putQMon wrong: %s"\
+                                               %st.putQMon
+        #First Item goes into buffer at t=0, second not (renege)?
+        assert st.bufferMon==[[0,0],[0,1]],"bufferMon wrong: %s"%st.bufferMon        
+       
+def makeStoreCompSuite():
+    suite = unittest.TestSuite()
+    ## Unmonitored Stores
+    testBasicTime  = makeStoreCompTestcase("testBasicTime")
+    testBasicEvent = makeStoreCompTestcase("testBasicEvent")
+    testBasicTimePut  = makeStoreCompTestcase("testBasicTimePut")
+    testBasicEventPut = makeStoreCompTestcase("testBasicEventPut")
+    ## Monitored Stores
+    testBasicTimePutM = makeStoreCompTestcase("testBasicTimePutM")
+    testBasicEventPutM = makeStoreCompTestcase("testBasicEventPutM")
+    
+    suite.addTests([testBasicTime,
+                    testBasicEvent,
+                    testBasicTimePut,
+                    testBasicEventPut,
+                    testBasicTimePutM,
+                    testBasicEventPutM]) 
+    return suite
+
+## ------------------------------------------------------------------
+##             
+##  Level: Tests for compound get               
+##             
+## ------------------------------------------------------------------
+class TBTLev(Process):
+    """Level: For testBasicTime"""
+    def tbt(self,level):
+        yield get,self,level,1
+        assert self.got,"did not get 1 unit"
+        yield (get,self,level,1),(hold,self,5)
+        if self.acquired(level):
+            assert self.got==1,"did not get 1 unit"
+        else:
+            assert not self.got and now()==5,"time renege not working"
+        
+class TBELev(Process):
+    """Level: For testBasicEvent"""
+    def tbe(self,level,trigger):
+        yield get,self,level,1
+        assert self.got,"did not get 1 unit"
+        yield (get,self,level,1),(waitevent,self,trigger)
+        if self.acquired(level):
+            assert self.got==1,"did not get 1 Item"
+        else:
+            assert now()==5.5 and self.eventsFired[0]==trigger,\
+                   "event renege not working"
+        
+class TBEtriggerLev(Process):
+    """Level: For testBasicEvent"""
+    def fire(self,trigger):
+        yield hold,self,5.5
+        trigger.signal()
+        
+class TBTLevPut(Process):
+    """Level: For testBasicTimePut"""
+    def tbt(self,level):
+        yield put,self,level,1
+        assert level.amount,"did not put 1 unit"
+        yield (put,self,level,1),(hold,self,5)
+        if self.stored(level):
+            assert False,"should have reneged"
+        else:
+            assert level.amount==1 and now()==5,"time renege not working" 
+
+class TBELevPut(Process):
+    """Level: For testBasicEventPut and testBasicEventPutM"""
+    def tbe(self,level,trigger):
+        yield (put,self,level,1),(waitevent,self,trigger)
+        if self.stored(level):
+            assert level.amount==1,"did not put 1 unit"
+        else:
+            assert False,"should not have reneged"
+        yield (put,self,level,1),(waitevent,self,trigger) 
+        if self.stored(level):
+            assert False, "should have reneged"
+        else:
+            assert now()==5.5 and self.eventsFired[0]==trigger ,\
+                   "renege not working"
+
+class TBEtriggerLevPut(Process):
+    """Level: For testBasicEventPut"""
+    def fire(self,trigger):
+        yield hold,self,5.5
+        trigger.signal()
+        
+class makeLevelCompTestcase(unittest.TestCase):
+    """Level: Testcase for compound get and put statements"""
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (get,self,level),(hold,self,time)"
+    ##                   == timeout renege
+    ##             for both unmonitored and monitored Levels
+    ## ------------------------------------------------------------------
+
+    def testBasicTime(self):
+        """Level (unmonitored): test 'yield (get,self,level),(hold,self,timeout)"""
+        initialize()
+        l=Level(initialBuffered=1)
+        t=TBTLev()
+        activate(t,t.tbt(level=l))
+        simulate(until=10)
+        
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (put,self,store),(hold,self,time)"
+    ##                   == timeout renege
+    ##             for both unmonitored and monitored Stores
+    ## ------------------------------------------------------------------
+    def testBasicTimePut(self):
+        """Level (unmonitored): 
+        test 'yield (put,self,level),(hold,self,timeout)"""
+        initialize()
+        l=Level(capacity=1)
+        t=TBTLevPut()
+        activate(t,t.tbt(level=l))
+        simulate(until=10)
+        
+           
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (get,self,store),(waitevent,self,event)"
+    ##                   == event renege
+    ##             for both unmonitored and monitored Levels
+    ## ------------------------------------------------------------------
+    def testBasicEvent(self):
+        """Level (unmonitored): 
+        test 'yield (get,self,level),(waitevent,self,event)"""
+        initialize()
+        l=Level(initialBuffered=1)
+        trig=SimEvent()
+        t=TBELev()
+        activate(t,t.tbe(level=l,trigger=trig))
+        tr=TBEtriggerLev()
+        activate(tr,tr.fire(trigger=trig))
+        simulate(until=10)
+        
+    def testBasicEventM(self):
+        """Level (monitored): 
+        test monitors with 'yield (get,self,level),(waitevent,self,event)"""
+        initialize()
+        l=Level(initialBuffered=1,monitored=True)
+        trig=SimEvent()
+        t=TBELev()
+        activate(t,t.tbe(level=l,trigger=trig))
+        tr=TBEtriggerLev()
+        activate(tr,tr.fire(trigger=trig))
+        simulate(until=10)
+        #First get (t=0) succeeded and second timed out at t=5.5?
+        assert l.getQMon==[[0,0],[0,1],[5.5,0]],"getQMon not working: %s"\
+                                               %l.getQMon
+        #Level amount incr. then decr. by 1 (t=0), 2nd get reneged at t=5.5?
+        assert l.bufferMon==[[0,1],[0,0]],\
+               "bufferMon not working: %s"%l.bufferMon
+                                            
+    ## ------------------------------------------------------------------
+    ##             TEST "yield (put,self,store),(waitevent,self,event)"
+    ##                   == event renege
+    ##             for both unmonitored and monitored Levels
+    ## ------------------------------------------------------------------
+    def testBasicEventPut(self):
+        """Level (unmonitored): 
+        test 'yield (put,self,level),(waitevent,self,event)"""
+        initialize()
+        l=Level(capacity=1)
+        trig=SimEvent()
+        t=TBELevPut()
+        activate(t,t.tbe(level=l,trigger=trig))
+        tr=TBEtriggerLevPut()
+        activate(tr,tr.fire(trigger=trig))
+        simulate(until=10)
+        
+    def testBasicEventPutM(self):
+        """Level (monitored): 
+        test monitors with 'yield (put,self,level),(waitevent,self,event)"""
+        initialize()
+        l=Level(capacity=1,monitored=True)
+        trig=SimEvent()
+        t=TBELevPut()
+        activate(t,t.tbe(level=l,trigger=trig))
+        tr=TBEtriggerLevPut()
+        activate(tr,tr.fire(trigger=trig))
+        simulate(until=10)
+        "First put succeeds, second reneges at t=5.5?"
+        assert l.putQMon==[[0,0],[0,1],[5.5,0]],"putQMon wrong: %s"\
+                                                %l.putQMon
+        "1 unit added at t=0, renege at t=5 before 2nd unit added?"
+        assert l.bufferMon==[[0,0],[0,1]],"bufferMon wrong: %s"%l.bufferMon
+
+def makeLevelCompSuite():
+    suite = unittest.TestSuite()
+    ## Unmonitored Levels
+    testBasicTime  = makeLevelCompTestcase("testBasicTime")
+    testBasicEvent = makeLevelCompTestcase("testBasicEvent")
+    testBasicTimePut = makeLevelCompTestcase("testBasicTimePut")
+    testBasicEventPut = makeLevelCompTestcase("testBasicEventPut")
+    ## Monitored Levels
+    testBasicEventM = makeLevelCompTestcase("testBasicEventM")
+    testBasicEventPutM = makeLevelCompTestcase("testBasicEventPutM")
+    
+    suite.addTests([testBasicTime,
+                    testBasicEvent,
+                    testBasicTimePut,
+                    testBasicEventPut,
+                    testBasicEventM,
+                    testBasicEventPutM]) 
+    return suite
 
 if __name__ == '__main__':
     alltests = unittest.TestSuite((makeSSuite(),makeRSuite(),
@@ -1761,7 +2194,9 @@ if __name__ == '__main__':
                                    makeESuite(),makeWSuite(),
                                    makeTOSuite(),makeEvtRenegeSuite(),
                                    makeLevelSuite(),
-                                   makeStoreSuite()
+                                   makeStoreSuite(),
+                                   makeStoreCompSuite(),
+                                   makeLevelCompSuite()
                                 ))
     runner = unittest.TextTestRunner()
     runner.run(alltests)
