@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-from __future__ import generators
 from SimPy.Simulation import *
 """CellularAutomata.py
 Simulation of two-dimensional cellular automata. Plays game of Life.
@@ -84,6 +82,7 @@ cells[(11,11)].state=True
 cells[(11,12)].state=True
 cells[(11,13)].state=True
 
+print 'CellularAutomata'
 s=Show()
 whenToStartShowing=10
 activate(s,s.picture(),delay=whenToStartShowing)
