@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-from __future__ import generators
 from Tkinter import *
 import tkMessageBox
 from SimPy.Simulation import *
@@ -141,4 +139,3 @@ BExit = Button(F,text="Quit",command=die).grid(row = 5, column = 1)
 F.pack()
 root.title("SiPy Model")
 root.mainloop()
-
