@@ -1,7 +1,6 @@
 #!/usr/bin/env python
-from __future__ import generators
 from SimPy.Simulation import *
-from random import uniform
+from random import uniform,seed
 import string
 
 def theTime(time):
@@ -55,7 +54,8 @@ class foobar_machine(Process):
             self.idle=1
             yield passivate,self
             
-print '\nModel: SimPy_worker_extend'
+seed(111333555)
+print 'SimPy_worker_extend'
 initialize()
 foo=foobar_machine()
 activate(foo,foo.foobar_Process(),delay=0)
