@@ -1,29 +1,25 @@
-#!/usr/bin/env python
-from __future__ import generators
-from SimPy.Simulation import *
-from random import Random,expovariate,uniform
-# sip095  erlang-B loss M/H/c/c, M/Ek/c/c, M/M/c/c limited
-#        $Author: kgmuller $
-# $Revision: 1.1.1.5 $  $Date: 2006/02/02 13:35:45 $
-# converted from pyt095 2002 Sept 10
-# 2002 9 20 included bcc function directly.
-"""
-Jobs (e.g messages) arrive randomly at rate $1.0$ per minute at a
-  2-server system.  Mean service time is $0.75$ minutes and the
+""" bcc.py
+
+  Queue with blocked customers cleared
+  Jobs (e.g messages) arrive randomly at rate 1.0 per minute at a
+  2-server system.  Mean service time is 0.75 minutes and the
   service-time distribution is (1) exponential, (2) Erlang-5, or (3)
-  hyperexponential with $p=1/8$,$m1=2.0$, and $m2=4/7$. However no
+  hyperexponential with p=1/8,m1=2.0, and m2=4/7. However no
   queue is allowed; a job arriving when all the servers are busy is
   rejected.
   
   Develop and run a simulation program to estimate the probability of
-  rejection (which, in steady-state, is the same as $p(c)$) Measure
+  rejection (which, in steady-state, is the same as p(c)) Measure
   and compare the probability for each service time distribution.
   Though you should test the program with a trace, running just a few
   jobs, the final runs should be of 10000 jobs without a trace. Stop
-  the simulation when 10000 jobs have been generated. Use the random
-  number stream 1 for all runs.
+  the simulation when 10000 jobs have been generated.
 """
-__version__ = '\nModel: BCC at 2-server system $Revision: 1.1.1.5 $ $Date: 2006/02/02 13:35:45 $'
+
+from SimPy.Simulation import *
+from random import seed,Random,expovariate,uniform
+
+## Model components ------------------------
 
 dist = ""
 def bcc(lam, mu,s):
@@ -53,7 +49,7 @@ def bcc(lam, mu,s):
     return {'lambda':lam,'mu':mu,'s':s,
 	'p0':p0,'p[i]':p,'ps':ps, 'L':L}
 
-def ErlangVariate(mean,K,g):
+def ErlangVariate(mean,K):
     """ Erlang random variate
 
     mean = mean
@@ -62,10 +58,10 @@ def ErlangVariate(mean,K,g):
     """
     sum = 0.0 ; mu = K/mean
     for i in range(K):
-        sum += g.expovariate(mu)
+        sum += expovariate(mu)
     return (sum)
 
-def HyperVariate(p,m1,m2,g):
+def HyperVariate(p,m1,m2):
     """ Hyperexponential random variate
 
     p = prob of branch 1
@@ -73,9 +69,9 @@ def HyperVariate(p,m1,m2,g):
     m2 = mean of exponential, branch 2
     g = rv to be used
     """
-    if g.random() < p:
-        return g.expovariate(1.0/m1)
-    else: return g.expovariate(1.0/m2)
+    if random() < p:
+        return expovariate(1.0/m1)
+    else: return expovariate(1.0/m2)
 
 def testHyperVariate():
     """ tests the HyerVariate rv generator"""
@@ -87,9 +83,7 @@ def testHyperVariate():
         x1 = HyperVariate(p[i],1.0,10.0,g)
         #print p[i], x1
         assert abs(x1 - x[i]) < 0.001,'HyperVariate error'
-    
 
-## use bcc instead!!
 def erlangB(rho,c):
     """ Erlang's B formula for probabilities in no-queue
 
@@ -114,20 +108,13 @@ def erlangB(rho,c):
 class JobGen(Process):
     """ generates a sequence of Jobs
     """    
-    def __init__(self, JobRate,MaxJob,mu):
-         Process.__init__(self)
-         self.JobRate = JobRate
-         self.MaxJob = MaxJob
-         self.g = Random(111333)
-         self.i = 0
-         self.mu =mu
      
-    def execute(self):
+    def execute(self,JobRate,MaxJob,mu):
          global NoInService, Busy
-         for i in range(self.MaxJob):         
-             j = Job(i,self.mu)
-             activate(j,j.execute(),delay=0.0)
-             t = self.g.expovariate(self.JobRate)
+         for i in range(MaxJob):         
+             j = Job()
+             activate(j,j.execute(i,mu),delay=0.0)
+             t = expovariate(JobRate)
              MT.tally(t)
              yield hold,self,t
          self.trace("Job generator finished")
@@ -138,41 +125,36 @@ class JobGen(Process):
 class Job(Process):
     """ Jobs that are either accepted or rejected
     """
-    g = Random(779977)
-    def __init__(self, i, mu):
-         Process.__init__(self)
-         self.mu = mu
-         self.i = i
      
-    def execute(self):
+    def execute(self,i,mu):
         """ Job execution, only if accepted"""
         global NoInService,Busy,dist,NoRejected
         if NoInService < c:
-            self.trace("Job %2d accepted b=%1d"%(self.i,Busy))
+            self.trace("Job %2d accepted b=%1d"%(i,Busy))
             NoInService +=1
             if NoInService == c:
                 Busy =1
                 try: BM.accum(Busy,now())
                 except: "accum error BM=",BM
             #yield   hold,self,Job.g.expovariate(self.mu);           dist= "Exponential"
-            yield   hold,self,ErlangVariate(1.0/self.mu,5,Job.g);          dist= "Erlang     "
+            yield   hold,self,ErlangVariate(1.0/mu,5);          dist= "Erlang     "
             #yield   hold,self,HyperVariate(1.0/8,m1=2.0,m2=4.0/7,g=Job.g); dist= "HyperExpon "
             NoInService -=1
             Busy =0
             BM.accum(Busy,now())
-            self.trace("Job %2d leaving b=%1d"%(self.i,Busy))
+            self.trace("Job %2d leaving b=%1d"%(i,Busy))
         else:
-            self.trace("Job %2d REJECT  b=%1d"%(self.i,Busy))
+            self.trace("Job %2d REJECT  b=%1d"%(i,Busy))
             NoRejected +=1
  
     def trace(self,message):
         if JobTRACING: print "%8.4f \t%s"%(now(), message)
 
 
-##--------------------------------------------------------------
+## Experiment data -------------------------
 c = 2
-lam = 1.0
-mu = 1.0/0.75
+lam = 1.0      ## per minute
+mu = 1.0/0.75  ## per minute
 p = 1.0/8 ; m1= 2.0;  m2 = 4.0/7.0
 K = 5
 rho = lam/mu
@@ -190,24 +172,23 @@ JobGenTRACING = 0
 BM=Monitor()
 MT = Monitor()
 
-def main():
-    initialize()
-    jbg = JobGen(1.0,JobMax,mu)
-    activate(jbg,jbg.execute(),0.0)
-    ## print now()
-    simulate(until=20000.0)
-    print "time at the end =",now()
-    ##print erlangB(rho,c)
-    print "now=",now(), " startTime ",BM.startTime
-    print "No Rejected = %d, ratio= %s"%(NoRejected,(1.0*NoRejected)/JobMax)
-    print "Busy proportion (%6s) = %8.6f"%(dist,BM.timeAverage(now()),)
-    print "Erlang pc (th)                = %8.6f"%(erlangB(rho,c)[c],)
+## Model/Experiment ------------------------------
 
-##--------------------------------------------------------------
+seed(111333)
+
+initialize()
+jbg = JobGen()
+activate(jbg,jbg.execute(1.0,JobMax,mu),0.0)
+simulate(until=20000.0)
+
+## Analysis/output -------------------------
+
+print 'bcc'
+print "time at the end =",now()
+print "now=",now(), " startTime ",BM.startTime
+print "No Rejected = %d, ratio= %s"%(NoRejected,(1.0*NoRejected)/JobMax)
+print "Busy proportion (%6s) = %8.6f"%(dist,BM.timeAverage(now()),)
+print "Erlang pc (th)                = %8.6f"%(erlangB(rho,c)[c],)
 
-if __name__ == '__main__':
-    print __version__
-    ## testHyperVariate()
-    main()
 
 
