@@ -1,10 +1,11 @@
-#!/usr/bin/env python
 from SimPy.Simulation import *
 import random
 """
-Demo of SimPy's event signalling synchronization constructs (introduced in SimPy 1.5)
+   Demo of SimPy's event signalling synchronization constructs
 """
 
+print 'demoSimPyEvents'
+
 ##Pavlov's dogs
 """Scenario:
 Dogs start to drool when Pavlov rings the bell.
@@ -22,6 +23,7 @@ class PavlovDog(Process):
             yield waitevent,self,bell
             print "%s %s drools"%(now(),self.name)
 
+random.seed(111333555)
 initialize()
 bell=SimEvent("bell")
 for i in range(4):
@@ -53,6 +55,7 @@ class TotalJob(Process):
             yield waitevent,self,e
         print now(),"All done"
 
+random.seed(111333555)
 initialize()
 allEvents=[]
 for i in range(10):
@@ -80,6 +83,7 @@ class Car(Process):
         ### End Critical Section
         intersectionFree.signal()
 
+random.seed(111333555)
 initialize()
 intersectionFree=SimEvent("Intersection free")
 intersectionFree.signal()
