@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from SimPy.SimulationStep import *
 import random
 
@@ -31,6 +30,8 @@ class Worker(Process):
                 yield release,self,item
             yield hold,self,2 #rest
     
+random.seed(111333555)
+print 'needResources'
 initialize()
 brush=Resource(capacity=1,name="brush")
 ladder=Resource(capacity=2,name="ladder")
